import autoBind from 'auto-bind';
export class BaseFetcherRenderer {
    constructor(visitor) {
        this.visitor = visitor;
        autoBind(this);
    }
    generateQueryVariablesSignature(hasRequiredVariables, operationVariablesTypes) {
        return `variables${hasRequiredVariables ? '' : '?'}: ${operationVariablesTypes}`;
    }
    generateInfiniteQueryVariablesSignature(hasRequiredVariables, operationVariablesTypes) {
        if (this.visitor.config.reactQueryVersion <= 4) {
            return `variables${hasRequiredVariables ? '' : '?'}: ${operationVariablesTypes}`;
        }
        return `variables: ${operationVariablesTypes}`;
    }
    generateInfiniteQueryOptionsSignature(infiniteQueryOptions, operationResultType) {
        if (this.visitor.config.reactQueryVersion <= 4) {
            return `options?: ${infiniteQueryOptions}<${operationResultType}, TError, TData>`;
        }
        return `options: Omit<${infiniteQueryOptions}<${operationResultType}, TError, TData>, 'queryKey'> & { queryKey?: ${infiniteQueryOptions}<${operationResultType}, TError, TData>['queryKey'] }`;
    }
    generateInfiniteQueryKey(node, hasRequiredVariables) {
        if (hasRequiredVariables)
            return `['${node.name.value}.infinite', variables]`;
        return `variables === undefined ? ['${node.name.value}.infinite'] : ['${node.name.value}.infinite', variables]`;
    }
    generateInfiniteQueryKeyMaker(node, operationName, operationVariablesTypes, hasRequiredVariables) {
        const signature = this.generateQueryVariablesSignature(hasRequiredVariables, operationVariablesTypes);
        return `\nuseInfinite${operationName}.getKey = (${signature}) => ${this.generateInfiniteQueryKey(node, hasRequiredVariables)};\n`;
    }
    generateInfiniteQueryRootKeyMaker(node, operationName) {
        return `\nuseInfinite${operationName}.rootKey = '${node.name.value}.infinite';\n`;
    }
    generateQueryKey(node, hasRequiredVariables) {
        if (hasRequiredVariables)
            return `['${node.name.value}', variables]`;
        return `variables === undefined ? ['${node.name.value}'] : ['${node.name.value}', variables]`;
    }
    generateQueryKeyMaker(node, operationName, operationVariablesTypes, hasRequiredVariables) {
        const signature = this.generateQueryVariablesSignature(hasRequiredVariables, operationVariablesTypes);
        return `\nuse${operationName}.getKey = (${signature}) => ${this.generateQueryKey(node, hasRequiredVariables)};\n`;
    }
    generateQueryRootKeyMaker(node, operationName) {
        return `\nuse${operationName}.rootKey = '${node.name.value}';\n`;
    }
    generateMutationKey(node) {
        return `['${node.name.value}']`;
    }
    generateMutationKeyMaker(node, operationName) {
        return `\nuse${operationName}.getKey = () => ${this.generateMutationKey(node)};\n`;
    }
    generateInfiniteQueryRequired() {
        if (this.visitor.config.reactQueryVersion <= 4) {
            return '?';
        }
        return '';
    }
    generateInfiniteQueryFormattedParameters(queryKey, queryFn) {
        if (this.visitor.config.reactQueryVersion <= 4) {
            return `${queryKey},
      ${queryFn},
      options`;
        }
        return `(() => {
    const { queryKey: optionsQueryKey, ...restOptions } = options;
    return {
      queryKey: optionsQueryKey ?? ${queryKey},
      queryFn: ${queryFn},
      ...restOptions
    }
  })()`;
    }
    generateQueryFormattedParameters(queryKey, queryFn) {
        if (this.visitor.config.reactQueryVersion <= 4) {
            return `${queryKey},
      ${queryFn},
      options`;
        }
        return `{
    queryKey: ${queryKey},
    queryFn: ${queryFn},
    ...options
  }`;
    }
    generateMutationFormattedParameters(mutationKey, mutationFn) {
        if (this.visitor.config.reactQueryVersion <= 4) {
            return `${mutationKey},
      ${mutationFn},
      options`;
        }
        return `{
    mutationKey: ${mutationKey},
    mutationFn: ${mutationFn},
    ...options
  }`;
    }
}
export class FetcherRenderer extends BaseFetcherRenderer {
}
